/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.io;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

public class ReadLine {
	
	public class SavePoint {
		private long position;
		private boolean dir;
		private SavePoint(long position, boolean dir) {
			this.position = position;
			this.dir = dir;
		}
		public String toString() {
			return "SavePoint("+position+", "+dir+")";
		}
	}

	private ReadRandom read;
	private long pos;
	private boolean dir; //true=forward & false=backward
	private long fileLength;
	private final long DIM = 100; //numero de bytes que andamos para tras para procurar o "lastLine"
	
	
	public ReadLine(File file) {
		try {
			read = new ReadRandom(file);
			pos = 0;
			dir = true;
			fileLength = file.length();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			read = null;
		}
	}
	
	public String getNextLine() {
		StringBuffer sb = new StringBuffer();
		
		int c;
		long p = pos;
		while((c = read.read(p)) != -1) {
			if ((char)c == '\r' || (char)c == '\n')
				break;
			sb.append((char)c);
			p++;
		}
		
		pos = p+1;
		
		if (!dir) {
			p = pos;
			while((c = read.read(p)) != -1) {
				if ((char)c == '\r' || (char)c == '\n')
					break;
				sb.append((char)c);
				p++;
			}
			
			pos = p+1;
		}
		
		if (c == -1)
			return null;
		
		dir = true;
		return sb.toString();
	}
	
	public String getPreviousLine() {
		StringBuffer sb = new StringBuffer();

		
		/**
		 * FIXME: endofline
		 */
		
		int c;
		long p = pos-2;
		while((c = read.read(p)) != -1) {
			if ((char)c == '\r' || (char)c == '\n')
				break;
			sb.append((char)c);
			p--;
		}
		
		if (dir) {
			sb = new StringBuffer();
			p--;
			while((c = read.read(p)) != -1) {
				if ((char)c == '\r' || (char)c == '\n')
					break;
				sb.append((char)c);
				p--;
			}
		}
		
		pos = p+1;
		
		dir = false;
		
		if (c == -1)
			return null;
		
		sb.reverse();
		
		return sb.toString();
	}
	
	public String getCurrentLine() {
		String line = null;
		if (dir) {
			getPreviousLine();
			line = getNextLine();
		}
		else {
			getNextLine();
			line = getPreviousLine();
			
		}
		return line;
	}
	
	public SavePoint setSavePoint() {
		return new SavePoint(pos, dir);
	}
	
	public void restoreSavePoint(SavePoint sp) {		
		this.pos = sp.position;
		this.dir = sp.dir;
	}
	
	
	public void close() {
		try {
			read.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public String getLastLine() {
		read.read(fileLength-1-DIM);
		pos = fileLength-1-DIM;
		
		while(	getNextLine()!= null) {}
		String line = getPreviousLine();
		return line;
	}
}
