/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.util.ArrayList;
import java.util.List;

import org.jfree.chart.JFreeChart;
import org.jfree.data.general.DefaultPieDataset;

import pt.unl.fct.di.jtraceview.plugins.sequences.FindConflictTransaction;
import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxAbortEvent;

public class AbortReasonChart implements MultipleChartType {

	private TraceFileReader reader;
	private ArrayList<JFreeChart> charts;
	private long falseC;
	private long trueC;
	private long doomedC;
	private long total;
	
	public AbortReasonChart() {
		total = 0;
		falseC = 0;
		doomedC = 0;
		trueC = 0;
	}
	
	
	
	@Override
	public ArrayList<JFreeChart> getCharts() {
		return charts;
	}


	@Override
	public void init(TraceFileReader traceFile) {
		reader = traceFile;
		reader.reset();
		Event e;
		
		List<Integer> threads = FindConflictTransaction.getThreads(reader);
		
		while((e = reader.getNextEvent()) != null) {

			if (e instanceof TxAbortEvent) {
				TxAbortEvent ev = (TxAbortEvent)e;
				if (ev.tid != -1) {
					
					if (ev.type == TxAbortEvent.USER) {
						continue;
					}
					else {
						if (FindConflictTransaction.getConflictTransaction2(ev, threads, reader) == 1) {
							trueC++;
						}
						else if (FindConflictTransaction.getConflictTransaction2(ev, threads, reader) == 2) {
							doomedC++;
						}	
						else {
							falseC++;
						}
					}	
				}
				total++;
			} 
		}
		
		DefaultPieDataset data = new DefaultPieDataset();
		data.setValue("Doomed Conflicts", (doomedC/(double)total)*100);
		data.setValue("False Conflicts", (falseC/(double)total)*100);
		data.setValue("True Conflicts", (trueC/(double)total)*100);
		
		JFreeChart chart = ChartCreator.createPieChart("Abort False Positives", data);
		charts = new ArrayList<JFreeChart>();
		charts.add(chart);
	}

}
