/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.NavigableMap;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.jfree.chart.JFreeChart;
import org.jfree.data.category.DefaultCategoryDataset;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxMemAccessEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxReadEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxWriteEvent;
import pt.unl.fct.di.jtraceview.util.Pair;


/**
 * Este Chart permite nos ver a utilizacao das varias celulas de memoria
 */
public class AccessMemChart implements MultipleChartType{
	
	private final int N_CELLS = 5;
	private TraceFileReader reader;
	private ArrayList<JFreeChart> charts;
	private	HashMap<String, Integer> cells;
	
	// Pair.first = read count, Pair.second = write count
	private	HashMap<String, Pair<Integer,Integer>> cellsRW;
		
	public AccessMemChart() {}
	
	@Override
	public ArrayList<JFreeChart> getCharts() {
		return charts;
	}
	
	@Override
	public void init(TraceFileReader traceFile) {

		reader = traceFile;
		reader.reset();
		
		cellsRW = new HashMap<String, Pair<Integer,Integer>>();
		cells = new HashMap<String, Integer>();

		Event e;
		
		// Map the number of times each memcell has been accessed
		while((e = reader.getNextEvent()) != null) {
			if(e instanceof TxMemAccessEvent){
				TxMemAccessEvent memEvent = (TxMemAccessEvent)e;
				String aux = memEvent.memCel;
				String memCell = aux.substring(aux.lastIndexOf(".")+1);
				
				if (e instanceof TxReadEvent)
					incrementCountPair(cellsRW, memCell,0);
				else if (e instanceof TxWriteEvent)
					incrementCountPair(cellsRW, memCell,1);
				else continue; // should never happen
				
				incrementCount(cells, memCell);
			}
		}
		
		// Order the results by the value (times accessed)
		TreeMap<Integer, String> orderedCells = new TreeMap<Integer, String>();
		Set<Entry<String, Integer>> set = cells.entrySet();
		
		for (Entry<String, Integer> entry : set) {
			// swap
			orderedCells.put(entry.getValue(), entry.getKey());
		}
		cells = null;

		// Descending order
		NavigableMap<Integer, String> tmpSet = orderedCells.descendingMap();
		Set<Entry<Integer, String>> orderedSet = tmpSet.entrySet();
		
		// Now create the Chart data
        DefaultCategoryDataset dataset = new DefaultCategoryDataset();
        int x = 0;
        String legend;

        for (Entry<Integer, String> entry : orderedSet) {
        	int value = entry.getKey();
        	legend = entry.getValue();
        	
        	dataset.addValue(value, "Total", legend);
        	Pair<Integer, Integer> rw = cellsRW.get(legend);
        	dataset.addValue(rw.first, "Read", legend);
        	dataset.addValue(rw.second, "Write", legend);
        	//System.out.println(legend + "\t"+ value);

        	x++; if (x>N_CELLS) break;
        }

        JFreeChart chart =
        	ChartCreator.create3DBarChart("MemCells","MemCell","Times accessed",
        										(DefaultCategoryDataset)dataset);
        
        charts = new ArrayList<JFreeChart>();
        charts.add(chart);
	}

	// Read: pos = 0 , Write: pos = 1
	private void incrementCountPair (HashMap<String, Pair<Integer,Integer>> hm, String key, int pos) {
		Pair<Integer,Integer> result = hm.get(key);
		if(result == null) {
			if (pos==0) result = new Pair<Integer,Integer>(1, 0);
			else result = new Pair<Integer,Integer>(0, 1);
		}
		else{
			if (pos==0) result.first = result.first+1;
			else result.second = result.second+1;
		}
		hm.put(key, result);
	}
	
	private void incrementCount (HashMap<String, Integer> hm, String key) {
		Integer result = hm.get(key);
		if(result == null)
			hm.put(key, 1);
		else
			hm.put(key, result+1);
	}
}
