/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.StandardCategoryItemLabelGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.data.category.DefaultCategoryDataset;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxCommitEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxStartEvent;

public class TransactionLifetimeChart implements MultipleChartType {

	private TraceFileReader reader;
	private ArrayList<JFreeChart> charts;
	private HashMap<Integer,Tx> time;
	private HashMap<Integer,Long> threads = new HashMap<Integer, Long>();
	
	private class Tx {
		public long maxtime = 0;
		public long totaltime = 0;
		public long mintime = Long.MAX_VALUE;
		public long total;
		
	}
	
	public TransactionLifetimeChart() {
		charts = null;
		time = new HashMap<Integer,Tx>();
	}
	
	@Override
	public ArrayList<JFreeChart> getCharts() {
		return charts;
	}

	@Override
	public void init(TraceFileReader traceFile) {
		reader = traceFile;
		reader.reset();
		
		Event e = reader.getCurrentEvent();
		
		if (e instanceof TxStartEvent) {
			threads.put(e.threadId, e.timestamp);
		}
		
		while((e = reader.getNextEvent()) != null) {
			if (e instanceof TxStartEvent) {
				threads.put(e.threadId, e.timestamp);
			}
			else if (e instanceof TxCommitEvent) {
				long tt;
				try
				{
					tt = e.timestamp-threads.get(e.threadId);
				}
				catch (Exception mye)
				{
					continue;
				}
				Tx tx = getTx(e.tid);
				if (tt > tx.maxtime) {
					tx.maxtime = tt;
				}
				if (tt < tx.mintime) {
					tx.mintime = tt;
				}
				tx.totaltime += tt;
				tx.total++;
			}
		}
		
		DefaultCategoryDataset data = generateDataset();
		
		JFreeChart chart = ChartFactory.createBarChart(
	            "Transaction Duration",       // chart title
	            "Individual Transactions",               // domain axis label
	            "Logic Time",                  // range axis label
	            data,                  // data
	            PlotOrientation.VERTICAL, // orientation
	            true,                     // include legend
	            true,                     // tooltips?
	            false                     // URLs?
	        );

		charts = new ArrayList<JFreeChart>();
		charts.add(chart);
		
		BarRenderer renderer = (BarRenderer)((CategoryPlot)chart.getPlot()).getRenderer();
		renderer.setBaseItemLabelGenerator(new StandardCategoryItemLabelGenerator());
		renderer.setBaseItemLabelsVisible(true);
		
	}
	
	private Tx getTx(int tid) {
		Tx tx = time.get(tid);
		if (tx == null) {
			tx = new Tx();
			time.put(tid, tx);
		}
		return tx;
	}
	
	
	private DefaultCategoryDataset generateDataset() {
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		
		for (Entry<Integer, Tx> e : time.entrySet()) {
			dataset.addValue(e.getValue().totaltime/e.getValue().total, 
					"Average Time", "Code "+e.getKey());
			dataset.addValue(e.getValue().maxtime, 
					"Max Time", "Code "+e.getKey());
			dataset.addValue(e.getValue().mintime, 
					"Min Time", "Code "+e.getKey());
		}
		
		
		return dataset;
	}
	

}

