/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.util.ArrayList;
import java.util.HashMap;

import org.jfree.chart.JFreeChart;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DefaultPieDataset;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxAbortEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxCommitEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxMemAccessEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxReadEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxStartEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxWriteEvent;

public class WastedWorkChart implements MultipleChartType {

	private TraceFileReader reader;
	private ArrayList<JFreeChart> charts;
	private long wasted;
	private long wastedR;
	private long wastedW;
	private long usefulR;
	private long usefulW;
	private long total;
	
	public WastedWorkChart() {
		wasted = 0;
		wastedR = 0; wastedW = 0;
		usefulR = 0; usefulW = 0;
		total = 0;
	}
	
	
	@Override
	public ArrayList<JFreeChart> getCharts() {
		return charts;
	}


	@Override
	public void init(TraceFileReader traceFile) {
		reader = traceFile;
		reader.reset();
		
		Event e;
		HashMap<Integer,Long> start = new HashMap<Integer,Long>();
		// Read = false, Write = true
		HashMap<Integer,Boolean> type = new HashMap<Integer,Boolean>();
		
		while((e = reader.getNextEvent()) != null) {
			if (e.tid != -1) {
				
				if (e instanceof TxStartEvent) {
					start.put(e.threadId, e.timestamp);
				}
				if (e instanceof TxMemAccessEvent) {
					if (e instanceof TxReadEvent)
						type.put(e.threadId, false);
					else if (e instanceof TxWriteEvent)
						type.put(e.threadId, true);
				}
				if (e instanceof TxAbortEvent) {
					Long l = start.get(e.threadId);
					if (l != null) {
						long count = (e.timestamp-l);
						wasted += count;
						total += count;
						
						if (type.get(e.threadId).booleanValue())
							wastedW += count;
						else wastedR += count;
					}
				}
				if (e instanceof TxCommitEvent) {
					Long l = start.get(e.threadId);
					if (l != null) {
						long count = (e.timestamp-l);
						total += count;
						
						if (type.get(e.threadId).booleanValue())
							usefulW += count;
						else usefulR += count;
					}
				}
			} 
		}

		charts = new ArrayList<JFreeChart>();

		charts.add(WastedBars());
		charts.add(WastedVSUseful());
		charts.add(UsefulReadVSUsefulWrite());
		charts.add(WastedReadVSWastedWrite());
	}

	private JFreeChart WastedBars() {
		// Now create the Chart data
        DefaultCategoryDataset dataset = new DefaultCategoryDataset();
        dataset.addValue(total-wasted, "Total Useful", "Useful");
        dataset.addValue(usefulR, "Useful Read", "Useful");
        dataset.addValue(usefulW, "Useful Write", "Useful");
        
        dataset.addValue(wasted, "Wasted Total", "Wasted");
        dataset.addValue(wastedR, "Wasted Read", "Wasted");
        dataset.addValue(wastedW, "Wasted Write", "Wasted");

        JFreeChart chart =
        	ChartCreator.create3DBarChart("Wasted Work","","Number of transactional operations",
        										(DefaultCategoryDataset)dataset);
        return chart;
	}
	
	private JFreeChart WastedVSUseful() {
		DefaultPieDataset data = new DefaultPieDataset();
		data.setValue("Wasted", (wasted/(double)total)*100);
		data.setValue("Useful", (100-(wasted/(double)total)*100));
		
		return ChartCreator.createPieChart("Wasted% vs Total%", data);
	}
	
	private JFreeChart UsefulReadVSUsefulWrite() {
		DefaultPieDataset data = new DefaultPieDataset();
		data.setValue("Useful Read Trans.", (usefulR/(double)(usefulR+usefulW))*100);
		data.setValue("Useful Write Trans.", (usefulW/(double)(usefulR+usefulW))*100);
		
		JFreeChart chart = ChartCreator.createPieChart("Useful Read% vs Useful Write%", data);
		return chart;
	}
	
	private JFreeChart WastedReadVSWastedWrite() {
		DefaultPieDataset data = new DefaultPieDataset();
		data.setValue("Wasted Read Trans.", (wastedR/(double)wasted)*100);
		data.setValue("Wasted Write Trans.", (wastedW/(double)wasted)*100);
		
		JFreeChart chart = ChartCreator.createPieChart("Wasted Read% vs Wasted Write%", data);
		return chart;
	}

}
