/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.sequences;

import java.util.HashMap;
import java.util.Map;


import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.events.TxAbortEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxCommitEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxStartEvent;
import pt.unl.fct.di.jtraceview.util.Pair;
import pt.unl.fct.di.jtraceview.view.tools.component.DefaultJThreadSequenceModel;
import pt.unl.fct.di.jtraceview.view.tools.component.JThreadSequenceModel;
import pt.unl.fct.di.jtraceview.view.tools.component.JThreadSequenceModel.Thread;
import pt.unl.fct.di.jtraceview.view.tools.component.JThreadSequenceModel.TimeSlice;

public class ThreadTransactionSequence implements ThreadSequence {

	private DefaultJThreadSequenceModel model;
	
	public ThreadTransactionSequence() {
		model = null;
	}
	
	@Override
	public JThreadSequenceModel getModel() {
		return model;
	}

	@Override
	public void init(TraceFileReader  traceFile) {
		TraceFileReader reader = traceFile;
		reader.reset();
		
		model = new DefaultJThreadSequenceModel();
		HashMap<Integer,Pair<Thread,TimeSlice>> threads = 
			new HashMap<Integer,Pair<Thread,TimeSlice>>(); 
		
		Event e = null;
		long offset = 0;
		boolean first = true;
		
		while((e = reader.getNextEvent()) != null) {
			
			
			if (e instanceof TxStartEvent) {
				if (e.tid != -1) {
					if (first) {
						offset = e.timestamp;
						first = false;
					}
					Pair<Thread,TimeSlice> p = getThread(threads, e.threadId);
					p.second = new TimeSlice("Tx "+e.tid, e.timestamp-offset, -1);
				}
			}
			else if (e instanceof TxCommitEvent) {
				if (e.tid != -1) {
					Pair<Thread,TimeSlice> p = getThread(threads, e.threadId);
					if (p.second == null) {
						//assert(false);
						//System.out.println("COMMIT WITHOUT START -> "+e);
						continue;
					}
					p.second.end = e.timestamp-offset;
					p.second.userData.put("last.event", e);
					p.first.addTimeSlice(p.second);
					p.second = null;
				}
			}
			else if (e instanceof TxAbortEvent) {
				if (((TxAbortEvent) e).tid != -1) {
					Pair<Thread,TimeSlice> p = getThread(threads, e.threadId);
					if (p.second == null) {
						//assert(false);
						//System.out.println("ABORT WITHOUT START -> "+e);
						continue;
					}
					p.second.end = e.timestamp-offset;
					p.second.id = ((TxAbortEvent) e).type.toString();
					p.second.userData.put("last.event", e);
					p.second.userData.put("savepoint", reader.setSavePoint());
					p.second.userData.put("offset", new Long(offset));
					p.first.addTimeSlice(p.second);
					p.second = null;
				}
			}
			
		}
		
		for (Map.Entry<Integer,Pair<Thread,TimeSlice>> t : threads.entrySet()) {
			model.addThread(t.getValue().first);
		}
		
	}

	private Pair<Thread,TimeSlice> getThread(HashMap<Integer, 
			Pair<Thread,TimeSlice>> threads, int threadId) {
		Pair<Thread,TimeSlice> res = null;
		res = threads.get(threadId);
		if (res == null) {
			res = new Pair<Thread,TimeSlice>(new Thread("Thread "+threadId), null);
			threads.put(threadId, res);
		}
		return res;
	}

}
