/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.util;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.List;

import org.jfree.chart.JFreeChart;

import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.DefaultFontMapper;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfTemplate;
import com.itextpdf.text.pdf.PdfWriter;

public class ExportChartToPDF {
	public static void exportPDF(JFreeChart chart) {
		int width = 640;
		int height = 480;
		// step 1
    	Document document = new Document(new Rectangle(width, height));
    	try {
    		// step 2
    		PdfWriter writer;
			writer = PdfWriter.getInstance(document, new FileOutputStream("chart.pdf"));
			// step 3
    		document.open();
    		// step 4
    		PdfContentByte cb = writer.getDirectContent();
    		PdfTemplate tp = cb.createTemplate(width, height);
    		Graphics2D g2d = tp.createGraphics(width, height, new DefaultFontMapper());
    		Rectangle2D r2d = new Rectangle2D.Double(0, 0, width, height);
    		chart.draw(g2d, r2d);
    		g2d.dispose();
    		cb.addTemplate(tp, 0, 0);
    	}
    	catch(DocumentException de) {
    		de.printStackTrace();
    	}
    	catch (FileNotFoundException e2) {
    		e2.printStackTrace();
    	}
    	// step 5
    	document.close();
	}
	
	private static final int DEFAULT_WIDTH = 640;
	private static final int DEFAULT_HEIGHT = 480;

	private static Dimension calculateDocumentDimension(int num, int cols) {
		Dimension d = new Dimension(0, 0);
		d.width = DEFAULT_WIDTH*cols;
		d.height = (num%cols == 0 ? num/cols : (num/cols)+1)*DEFAULT_HEIGHT;
		return d;
	}
	
	public static void exportPDF(List<JFreeChart> charts, int cols, String filename) {
		Dimension dim = calculateDocumentDimension(charts.size(), cols);
		// step 1
    	Document document = new Document(new Rectangle(dim.width, dim.height));
    	try {
    		// step 2
    		PdfWriter writer;
			writer = PdfWriter.getInstance(document, new FileOutputStream(filename+".pdf"));
			// step 3
    		document.open();
    		// step 4
    		PdfContentByte cb = writer.getDirectContent();
    		PdfTemplate tp = cb.createTemplate(dim.width, dim.height);
    		Graphics2D g2d = tp.createGraphics(dim.width, dim.height, new DefaultFontMapper());
    		int y = 0;
    		int x = 0;
    		int i = 0;
    		for (JFreeChart chart : charts) {
    			Rectangle2D r2d = new Rectangle2D.Double(x, 
    						y, DEFAULT_WIDTH, DEFAULT_HEIGHT);
    			chart.draw(g2d, r2d);
    			i++;
    			if (i%2 == 0) {
    				y += DEFAULT_HEIGHT;
    				x = 0;
    			}
    			else {
    				x += DEFAULT_WIDTH;
    			}
    		}
    		g2d.dispose();
    		cb.addTemplate(tp, 0, 0);
    	}
    	catch(DocumentException de) {
    		de.printStackTrace();
    	}
    	catch (FileNotFoundException e2) {
    		e2.printStackTrace();
    	}
    	// step 5
    	document.close();
	}
}
