/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.util;

import java.io.File;
import java.util.ArrayList;

import org.jfree.chart.JFreeChart;

import pt.unl.fct.di.jtraceview.plugins.charts.AbortChart;
import pt.unl.fct.di.jtraceview.plugins.charts.AbortReasonChart;
import pt.unl.fct.di.jtraceview.plugins.charts.AccessMemChart;
import pt.unl.fct.di.jtraceview.plugins.charts.CommitAbortChart;
import pt.unl.fct.di.jtraceview.plugins.charts.CommitAbortXYChart;
import pt.unl.fct.di.jtraceview.plugins.charts.MultipleChartType;
import pt.unl.fct.di.jtraceview.plugins.charts.ReadWriteTransactionChart;
import pt.unl.fct.di.jtraceview.plugins.charts.TransactionIdChart;
import pt.unl.fct.di.jtraceview.plugins.charts.TransactionRetryChart;
import pt.unl.fct.di.jtraceview.plugins.charts.WastedWorkChart;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;

public class TraceToPDF {

	public static void main(String[] args) {
		
		if (args.length < 2) {
			System.out.println("Usage: TraceToPDF src_dir dest_dir");
			System.exit(1);
		}
		
		File src = new File(args[0]);
		File dest = new File(args[1]);
		if (!src.isDirectory() || !dest.isDirectory()) {
			System.out.println("The pathname must be a directory");
			System.exit(1);
		}
		
		ArrayList<MultipleChartType> multipleChart;
		multipleChart = new ArrayList<MultipleChartType>();
				
		multipleChart.add(new AbortChart());
		multipleChart.add(new AbortReasonChart());
		multipleChart.add(new AccessMemChart());
		multipleChart.add(new CommitAbortChart());
		multipleChart.add(new CommitAbortXYChart());
		multipleChart.add(new ReadWriteTransactionChart());
		multipleChart.add(new TransactionIdChart());
		multipleChart.add(new TransactionRetryChart());
		multipleChart.add(new WastedWorkChart());
		
		int countF = 0; // number of files processed
		int countC = 0; // number of charts processed
		int step = (int)(1.0/multipleChart.size()*100);
		
		// Get current time
		long start = System.currentTimeMillis();
		
		for (File f : src.listFiles()) {
			if (!f.isDirectory() && f.getName().endsWith(".bin")) {
				
				String fileName = f.getName();
				int index = fileName.lastIndexOf(".bin");
				fileName = fileName.substring(0, index);

				System.out.print("Processing..");
				
				ArrayList<JFreeChart> charts = new ArrayList<JFreeChart>();
				
				countC = 0;
				TraceFileReader tfr = new TraceFileReader(f);

				for (MultipleChartType c : multipleChart) {
					System.out.print(" "+countC*step+"%");
//					long startC = System.currentTimeMillis();
					c.init(tfr);
					charts.addAll(c.getCharts());
					countC++;
//					System.out.println("Took: "+(System.currentTimeMillis()-startC)+" milliseconds.");
				}
				
				System.out.println(" 100%");
				
				ExportChartToPDF.exportPDF(charts, 2, dest.getAbsolutePath()+"/"+fileName);
				
				countF++;
			}
		}
		// Get elapsed time in milliseconds
		long elapsedTimeMillis = System.currentTimeMillis()-start;
		// Get elapsed time in seconds
		float elapsedTimeSec = elapsedTimeMillis/1000F;
		
		System.out.println(countF + " trace file(s) exported to pdf in "+elapsedTimeSec+" seconds");

	}

}
