/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.view;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.net.URL;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;

import org.jfree.chart.JFreeChart;
import org.jfree.ui.ExtensionFileFilter;

import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.util.ExportChartToPDF;
import pt.unl.fct.di.jtraceview.view.tools.ChartsViewerPanel;
import pt.unl.fct.di.jtraceview.view.tools.ThreadSequencePanel;
import pt.unl.fct.di.jtraceview.view.tools.component.JThreadSequence;
import pt.unl.fct.di.jtraceview.view.tree.ChartGroupTreeItem;
import pt.unl.fct.di.jtraceview.view.tree.MultipleChartTreeItem;
import pt.unl.fct.di.jtraceview.view.tree.ToolsTree;

public class MainForm extends JFrame {

	public static final int CHART_MODE = 1;

	private static MainForm instance = null;

	public static final int SEQUENCE_MODE = 0;

	private static final long serialVersionUID = -589587365280450774L;

	public static synchronized MainForm getMainForm() {
		if (instance == null) {
			instance = new MainForm();
			instance.setTitle("JTraceView");
			instance.setVisible(true);
		}

		return instance;
	}

	public static void main(String[] args) {

		try {

			// Set System L&F
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (UnsupportedLookAndFeelException e) {
			// handle exception
		} catch (ClassNotFoundException e) {
			// handle exception
		} catch (InstantiationException e) {
			// handle exception
		} catch (IllegalAccessException e) {
			// handle exception
		}

		// MainForm mf = new MainForm();
		// mf.pack();
		// mf.setVisible(true);

		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				MainForm.getMainForm();
			}
		});

	}

	private JPanel centerPanel;
	private JPanel controlPanel;
	
	public JPanel getControlPanel() {
		return controlPanel;
	}

	private ChartsViewerPanel chartPanel;
	private int currentMode;

	private JProgressBar progressBar;
	private ThreadSequencePanel sequencePanel;
	private JThreadSequence sequence;
	private File traceFile = null;
	private TraceFileReader traceReader = null;

	private JTree tree;
	
	

	public JThreadSequence getSequence() {
		return sequence;
	}

	public MainForm() {
		this.setSize(1024, 760);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setLayout(new BorderLayout());

		currentMode = CHART_MODE;

		JMenuBar bar = new JMenuBar();
		JMenu menu = new JMenu("File");
		bar.add(menu);
		String imgLocation = "/jlfgr-1_0/toolbarButtonGraphics/general/Open16.gif";
		URL imageURL = MainForm.class.getResource(imgLocation);
		JMenuItem open = new JMenuItem("Open", new ImageIcon(imageURL));
		open.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				openTraceFile();
			}
		});
		menu.add(open);
		
		JMenuItem pdf = new JMenuItem("Export PDF", null);
		pdf.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				//System.out.println(tree.getSelectionPath().getLastPathComponent().getClass());
				if (tree==null) return;
				Object o = tree.getSelectionPath().getLastPathComponent();
				if (o instanceof MultipleChartTreeItem) {
					MultipleChartTreeItem item = (MultipleChartTreeItem)o;
					List<JFreeChart> charts = item.getBaseCharts(traceReader);
					String fileName = traceFile.getName();
					fileName = fileName.substring(0, fileName.lastIndexOf(".bin"));
					ExportChartToPDF.exportPDF(charts, 2, fileName);
				}
				else if (o instanceof ChartGroupTreeItem) {
					List<JFreeChart> charts = chartPanel.getChartList();
					if (charts != null) {
						String fileName = traceFile.getName();
						fileName = fileName.substring(0, fileName.lastIndexOf(".bin"));
						ExportChartToPDF.exportPDF(charts, 2, fileName);
					}
				}
			}
		});
		menu.add(pdf);

		this.setJMenuBar(bar);

		JToolBar tb = new JToolBar("SideBar", JToolBar.VERTICAL);
		tb.setFloatable(false);
		tb.setMargin(new Insets(5, 5, 5, 5));

		// tb.add(new CommitAbortItem());
		// tb.addSeparator();
		// tb.add(new AbortItem());
		// tb.addSeparator();

		// DefaultMutableTreeNode root = new DefaultMutableTreeNode("Charts");
		//		
		// root.add(new DefaultMutableTreeNode("Commit/Abort"));
		// root.add(new DefaultMutableTreeNode("Abort Types"));

		// tree = new JTree(root);
		// tree.getSelectionModel().setSelectionMode
		// (TreeSelectionModel.SINGLE_TREE_SELECTION);
		// tree.addTreeSelectionListener(new TreeSelectionListener() {
		// @Override
		// public void valueChanged(TreeSelectionEvent e) {
		// DefaultMutableTreeNode node = (DefaultMutableTreeNode)
		// tree.getLastSelectedPathComponent();
		// if (node == null) return;
		//				
		// System.out.println(node);
		// }
		// });

		tree = new ToolsTree();

		JScrollPane treeView = new JScrollPane(tree);
		tb.add(treeView);

		this.getContentPane().add(tb, BorderLayout.WEST);
		
		/**
		 * Control Panel for controlling info of Thread Sequences
		 */
		controlPanel = new JPanel();
		controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.Y_AXIS));
		
		JLabel zoomLabel = new JLabel("Sequence Zoom", JLabel.CENTER);
		zoomLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		JSlider zoom = new JSlider(1, 10000, 5000);
		JCheckBox box = new JCheckBox("Enable/Disable arrows");
		JRadioButton color = new JRadioButton("Colored");
		JRadioButton blackwhite = new JRadioButton("Black&White");
		ButtonGroup radioButtonGroup = new ButtonGroup();
		color.setSelected(true);
		radioButtonGroup.add(color);
		radioButtonGroup.add(blackwhite);
	    color.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e){
				if(e.getStateChange() == ItemEvent.SELECTED)
					sequence.setColor();
				sequence.repaint();	
			}
		});
	    blackwhite.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e){
				if(e.getStateChange() == ItemEvent.SELECTED)
					sequence.setBlackWhite();
				sequence.repaint();					
			}
		});
		box.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent event) {
				sequence.setSelected(event.getStateChange() == ItemEvent.SELECTED);
//				sequence.updateUI();	Se deixarmos isto ele faz sempre o compute dos Arrows
				sequence.repaint();
			}

		});
		// zoom.setMajorTickSpacing(100);
		// zoom.setMinorTickSpacing(50);
		// zoom.setPaintTicks(true);
		// zoom.setPaintLabels(true);

		zoom.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				sequence.setTimeSliceWidthFactor(1.0 / (double) source.getValue());
				sequence.updateUI();
				// sequence.invalidate();
				sequence.repaint();
			}

		});
		
		controlPanel.add(zoomLabel);
		controlPanel.add(zoom);
		controlPanel.add(box);
		controlPanel.add(color);
		controlPanel.add(blackwhite);
		
		controlPanel.setVisible(false);
		
		tb.add(controlPanel);
		
		///////////////////////////////////////////////////////////

		chartPanel = new ChartsViewerPanel();
		sequencePanel = new ThreadSequencePanel();
		sequence = new JThreadSequence();
		
		sequencePanel.add(sequence);

		CardLayout cl = new CardLayout();
		centerPanel = new JPanel(cl);

		centerPanel.add(new JScrollPane(chartPanel), "ChartPanel");
		//centerPanel.add(new JScrollPane(sequencePanel), "SequencePanel");
		centerPanel.add(new JScrollPane(sequence), "SequencePanel");

		this.getContentPane().add(centerPanel, BorderLayout.CENTER);

		progressBar = new JProgressBar();
		this.getContentPane().add(progressBar, BorderLayout.SOUTH);
		
		openTraceFile();
	}

	public ChartsViewerPanel getChartPanel() {
		return chartPanel;
	}

	public JProgressBar getProgressBar() {
		return progressBar;
	}

	public ThreadSequencePanel getSequencePanel() {
		return sequencePanel;
	}

	public File getTraceFile() {
		return traceFile;
	}
	
	public TraceFileReader getTraceReader() {
		return traceReader;
	}

	public JTree getTree() {
		return tree;
	}

	protected JButton makeNavigationButton(String img) {
		// Look for the image.
		String imgLocation = "/jlfgr-1_0/toolbarButtonGraphics/general/" + img;
		URL imageURL = MainForm.class.getResource(imgLocation);

		ImageIcon icon = new ImageIcon(imageURL);
		JButton button = new JButton(icon);
		button.setHorizontalAlignment(SwingConstants.CENTER);
		button.setVerticalTextPosition(SwingConstants.BOTTOM);
		button.setHorizontalTextPosition(SwingConstants.CENTER);

		return button;
	}

	public void openTraceFile() {
		JFileChooser file = new JFileChooser(".");
		FileFilter filter = new ExtensionFileFilter("Trace Files", "bin");
		file.setFileFilter(filter);
		int ret = file.showOpenDialog(file);

		if (ret == JFileChooser.APPROVE_OPTION) {
			File f = file.getSelectedFile();
			traceFile = f;
			setTitle("Trace File: " + f.getName());
			
			// close the old one
			if (traceReader!=null)
				traceReader.close();
			// open the new one
			traceReader = new TraceFileReader(traceFile);
		}
	}

	public void setMode(int mode) {
		if (currentMode != mode) {
			CardLayout cl = (CardLayout) centerPanel.getLayout();
			if (mode == SEQUENCE_MODE) {
				cl.show(centerPanel, "SequencePanel");
			} else {
				cl.show(centerPanel, "ChartPanel");
			}
			currentMode = mode;

		}
	}
}
