/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.view.tree;

import java.awt.Cursor;
import java.io.File;

import javax.swing.SwingWorker;

import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.view.MainForm;
import pt.unl.fct.di.jtraceview.view.tools.component.JThreadSequenceModel;

@SuppressWarnings("serial")
public abstract class SequenceTreeItem extends TreeItem {

	protected File currentFile; 
	protected File previousFile;
	protected TraceFileReader currentReader;
	protected JThreadSequenceModel sequenceCache;
	
	public SequenceTreeItem(String title) {
		super(title);
		this.currentFile = null;
		this.previousFile = null;
		this.sequenceCache = null;
	}

	@Override
	public void itemSelected() {
		if (MainForm.getMainForm().getTraceFile() == null)
			MainForm.getMainForm().openTraceFile();
		
		(new SwingWorker<JThreadSequenceModel,Object>() {
			@Override
			protected JThreadSequenceModel doInBackground() throws Exception {
				MainForm.getMainForm().getProgressBar().setIndeterminate(true);
				MainForm.getMainForm().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				currentFile = MainForm.getMainForm().getTraceFile();
				currentReader = MainForm.getMainForm().getTraceReader();
				if (sequenceCache == null || changedFile()) {
					sequenceCache = getSequence(currentReader);
					previousFile = currentFile;
				}
				
				return sequenceCache;
			}
			
			@Override
			protected void done() {
				try { 
					MainForm.getMainForm().getSequence().setModel(get());
					MainForm.getMainForm().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					MainForm.getMainForm().getProgressBar().setIndeterminate(false);
					MainForm.getMainForm().setMode(MainForm.SEQUENCE_MODE);
					MainForm.getMainForm().getControlPanel().setVisible(true);
		        } catch (Exception ignore){}
			}

		}).execute();
	}
	
	protected boolean changedFile() {
		return currentFile != previousFile;
	}
	
	public abstract JThreadSequenceModel getSequence(TraceFileReader traceFile);

}
