/*  Copyright (C) 2010 Faculdade de Ciências e Tecnologia 
 *                     Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdint.h>
#include <string.h>
#include <inttypes.h>
#include <stdio.h>
#include "event.h"
#include "xalloc.h"


const char const *tx_type_string[]={"","start","commit","abort","read","write"};
const char const *tx_abort_type_string[]={"","user","commit","other"};


struct event *
event_new(void)
{
	return xmalloc(sizeof(struct event));
}

struct event *
event_new_set(char *raw)
{
	struct event *n=event_new();

	event_set(n,raw);

	return n;
}

void
event_set(struct event *e, char *raw)
{
	char *raw_current=raw;

	memcpy(&e->timestamp,raw_current,sizeof(e->timestamp));
	raw_current+=sizeof(e->timestamp);

	memcpy(&e->id,raw_current,sizeof(e->id));
	raw_current+=sizeof(e->id);

	memcpy(&e->thread_id,raw_current,sizeof(e->thread_id));
	raw_current+=sizeof(e->thread_id);

	memcpy(&e->tx_id,raw_current,sizeof(e->tx_id));
	raw_current+=sizeof(e->tx_id);

	memcpy(&e->extra,raw_current,sizeof(e->extra));
	raw_current+=sizeof(e->extra);
}

char *event_get(struct event *e) // the return value must be freed
{
	char *raw=xmalloc(EVENT_SIZE);
	char *raw_current=raw;

	memcpy(raw_current,&e->timestamp,sizeof(e->timestamp));
	raw_current+=sizeof(e->timestamp);

	memcpy(raw_current,&e->id,sizeof(e->id));
	raw_current+=sizeof(e->id);	

	memcpy(raw_current,&e->thread_id,sizeof(e->thread_id));
	raw_current+=sizeof(e->thread_id);

	memcpy(raw_current,&e->tx_id,sizeof(e->tx_id));
	raw_current+=sizeof(e->tx_id);

	memcpy(raw_current,&e->extra,sizeof(e->extra));

	return raw;
}

void
event_free(struct event *e)
{
	free(e);
}

void
event_print(struct event *e)
{
	printf("%"PRIu64,e->timestamp);
	printf("\t%s",tx_type_string[e->id]);
	printf("\tt%"PRIu16 "\t",e->thread_id);
	printf("\ttx%"PRIu8 "\t",e->tx_id);

	switch (e->id)
	{
	case TX_READ:
	case TX_WRITE: printf("\t0x%"PRIx64,e->extra.addr); break;
	case TX_ABORT: printf("\t%s",
			      tx_abort_type_string[e->extra.abort_type]); 
		break;
	}

	putchar('\n');
}
