/*  Copyright (C) 2010 Faculdade de Ciências e Tecnologia 
 *                     Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#include "../common/event.h"
#include "../common/xalloc.h"

#define BUFFER_SIZE 1024

int
analyse_file(char *filename)
{
	struct event *e=event_new();
	int in;
	char *raw;
	int r;
	int stats[6];
	
	raw=xmalloc(EVENT_SIZE*BUFFER_SIZE);
	
	in=open(filename,O_RDONLY);
	
	if (in < 0)
	{
		perror("open()");
		return -1;
	}

	memset(stats,0,sizeof(stats));
	
	while ((r=read(in,raw,EVENT_SIZE*BUFFER_SIZE)))
	{
		int events=r/EVENT_SIZE;
	
		for (int i=0; i < events; i++)
		{
                        event_set(e,raw+EVENT_SIZE*i);
			stats[e->id]++;
		}
	}

	printf("Stats about %s:\n",filename);

	for (int i=1; i < 6; i++)
		printf("\tTotal %s: %d\n",tx_type_string[i],stats[i]);

	event_free(e);
	free(raw);
	close(in);

	return 0;
}

int
main(int argc, char **argv)
{
	if (argc < 2)
	{
		fprintf(stderr,"Usage: %s <trace_file>+\n",
			argv[0]);
		return -1;
	}

	for (int i=1; argv[i]; i++)
	{
		analyse_file(argv[i]);
		if (argv[i+1])
			putchar('\n');
	}

	return 0;
}
